/**
 * Base class for search strategies with common functionality
 */
export class BaseSearchStrategy {
    config;
    constructor(config) {
        this.config = config;
    }
    /**
     * Exact search implementation for backward compatibility
     */
    exactSearch(query, entities) {
        const lowerQuery = query.toLowerCase();
        return entities.filter(e => {
            const entityTags = e.tags || [];
            return (e.name.toLowerCase().includes(lowerQuery) ||
                e.entityType.toLowerCase().includes(lowerQuery) ||
                e.observations.some(o => o.toLowerCase().includes(lowerQuery)) ||
                entityTags.some(tag => tag.toLowerCase().includes(lowerQuery)));
        });
    }
    /**
     * Filter entities by exact tags
     */
    filterByExactTags(entities, exactTags, tagMatchMode = 'any') {
        return entities.filter(entity => {
            const entityTags = entity.tags || [];
            if (tagMatchMode === 'all') {
                return exactTags.every(tag => entityTags.includes(tag));
            }
            else {
                return exactTags.some(tag => entityTags.includes(tag));
            }
        });
    }
    /**
     * Helper method to handle multiple queries with deduplication
     */
    async searchMultipleDatabase(queries, threshold, project) {
        let allResults = [];
        const existingEntityNames = new Set();
        for (const query of queries) {
            const results = await this.searchDatabase(query, threshold, project);
            // Add only new entities (deduplication)
            const newEntities = results.filter(e => !existingEntityNames.has(e.name));
            allResults.push(...newEntities);
            // Update the set of existing entity names
            newEntities.forEach(e => existingEntityNames.add(e.name));
        }
        return allResults;
    }
    /**
     * Helper method to handle multiple queries for client-side search with deduplication
     */
    searchMultipleClientSide(entities, queries) {
        let allResults = [];
        const existingEntityNames = new Set();
        for (const query of queries) {
            const results = this.searchClientSide(entities, query);
            // Add only new entities (deduplication)
            const newEntities = results.filter(e => !existingEntityNames.has(e.name));
            allResults.push(...newEntities);
            // Update the set of existing entity names
            newEntities.forEach(e => existingEntityNames.add(e.name));
        }
        return allResults;
    }
}
//# sourceMappingURL=base-strategy.js.map