import Fuse from 'fuse.js';
import { BaseSearchStrategy } from './base-strategy.js';
import { getValidatedSearchLimits } from '../config.js';
/**
 * PostgreSQL search strategy - supports both database-level and client-side fuzzy search
 */
export class PostgreSQLFuzzyStrategy extends BaseSearchStrategy {
    pgPool;
    project;
    searchLimits = getValidatedSearchLimits();
    constructor(config, pgPool, project) {
        super(config);
        this.pgPool = pgPool;
        this.project = project;
    }
    canUseDatabase() {
        return this.config.useDatabaseSearch;
    }
    async searchDatabase(query, threshold, project) {
        // Handle multiple queries with optimized SQL
        if (Array.isArray(query)) {
            return this.searchMultipleDatabaseOptimized(query, threshold, project);
        }
        // Single query - use existing logic
        return this.searchSingleDatabase(query, threshold, project);
    }
    async searchSingleDatabase(query, threshold, project) {
        const client = await this.pgPool.connect();
        try {
            // Use provided project parameter or fall back to constructor project
            const searchProject = project || this.project;
            const result = await client.query(`
        SELECT e.*,
               GREATEST(
                 similarity(e.name, $1),
                 similarity(e.entity_type, $1),
                 similarity(e.observations::text, $1),
                 similarity(e.tags::text, $1)
               ) as relevance_score
        FROM entities e
        WHERE e.project = $3
          AND (similarity(e.name, $1) > $2
               OR similarity(e.entity_type, $1) > $2
               OR similarity(e.observations::text, $1) > $2
               OR similarity(e.tags::text, $1) > $2)
        ORDER BY relevance_score DESC
        LIMIT $4
      `, [query, threshold, searchProject, this.searchLimits.maxResults]);
            return result.rows.map(row => ({
                name: row.name,
                entityType: row.entity_type,
                observations: row.observations || [],
                tags: row.tags || []
            }));
        }
        finally {
            client.release();
        }
    }
    searchClientSide(entities, query) {
        // Handle multiple queries
        if (Array.isArray(query)) {
            return this.searchMultipleClientSide(entities, query);
        }
        // Single query - use existing logic
        return this.searchSingleClientSide(entities, query);
    }
    searchSingleClientSide(entities, query) {
        const fuseOptions = {
            threshold: this.config.threshold,
            distance: 100,
            includeScore: true,
            keys: ['name', 'entityType', 'observations', 'tags'],
            ...this.config.fuseOptions
        };
        const fuse = new Fuse(entities, fuseOptions);
        const results = fuse.search(query);
        return results.map(result => result.item);
    }
    /**
     * Optimized multiple query search using single SQL query with OR conditions
     * This replaces the inefficient sequential query processing from base strategy
     */
    async searchMultipleDatabaseOptimized(queries, threshold, project) {
        // Handle empty queries array
        if (queries.length === 0) {
            return [];
        }
        // For very large query arrays, use batching to avoid excessive SQL complexity
        if (queries.length > this.searchLimits.batchSize) {
            return this.searchMultipleDatabaseBatched(queries, threshold, project, this.searchLimits.batchSize);
        }
        const client = await this.pgPool.connect();
        try {
            const searchProject = project || this.project;
            // Build parameterized query with OR conditions for each search term
            const conditions = queries.map((_, index) => `(similarity(e.name, $${index + 1}) > $${queries.length + 1} OR
          similarity(e.entity_type, $${index + 1}) > $${queries.length + 1} OR
          similarity(e.observations::text, $${index + 1}) > $${queries.length + 1} OR
          similarity(e.tags::text, $${index + 1}) > $${queries.length + 1})`).join(' OR ');
            // Build relevance score calculation for all queries
            const relevanceCalculations = queries.map((_, index) => `similarity(e.name, $${index + 1}),
         similarity(e.entity_type, $${index + 1}),
         similarity(e.observations::text, $${index + 1}),
         similarity(e.tags::text, $${index + 1})`).join(', ');
            const result = await client.query(`
        SELECT DISTINCT e.*,
               GREATEST(${relevanceCalculations}) as relevance_score
        FROM entities e
        WHERE e.project = $${queries.length + 2}
          AND (${conditions})
        ORDER BY relevance_score DESC
        LIMIT $${queries.length + 3}
      `, [...queries, threshold, searchProject, this.searchLimits.maxResults]);
            return result.rows.map(row => ({
                name: row.name,
                entityType: row.entity_type,
                observations: row.observations || [],
                tags: row.tags || []
            }));
        }
        finally {
            client.release();
        }
    }
    /**
     * Handle very large query arrays by processing them in batches
     */
    async searchMultipleDatabaseBatched(queries, threshold, project, batchSize = 10) {
        let allResults = [];
        const existingEntityNames = new Set();
        // Process queries in batches
        for (let i = 0; i < queries.length; i += batchSize) {
            const batchQueries = queries.slice(i, i + batchSize);
            // Use optimized SQL for this batch
            const batchResults = await this.searchMultipleDatabaseOptimized(batchQueries, threshold, project);
            // Deduplicate results
            const newEntities = batchResults.filter(e => !existingEntityNames.has(e.name));
            allResults.push(...newEntities);
            newEntities.forEach(e => existingEntityNames.add(e.name));
        }
        return allResults;
    }
}
//# sourceMappingURL=postgresql-strategy.js.map